import logging
from re import match, compile
from ortu.oci_auth import get_network_client
from oci.core.models import UpdateRouteTableDetails, RouteRule


_log = logging.getLogger(name=__name__)


CIDR_DESTINATION_TYPE = "CIDR_BLOCK"
SERVICE_DESTINATION_TYPE = "SERVICE_CIDR_BLOCK"
CIDR_RE_PATTERN = compile('(?<!\d\.)(?<!\d)(?:\d{1,3}\.){3}\d{1,3}\/\d{1,2}(?!\d|(?:\.\d))')


def _construct_route_rule(cidr, ne_ocid):
    if match(CIDR_RE_PATTERN, cidr):
        route_rule = RouteRule(
            destination=cidr,
            destination_type=CIDR_DESTINATION_TYPE,
            network_entity_id=ne_ocid
        )
    else:
        route_rule = RouteRule(
            destination=cidr,
            destination_type=SERVICE_DESTINATION_TYPE,
            network_entity_id=ne_ocid
        )

    return route_rule


def get_route_table(rt_ocid):
    network_client = get_network_client()

    return network_client.get_route_table(rt_ocid).data


def check_route(route_table, cidr, ne_ocid):
    route_rule = _construct_route_rule(cidr, ne_ocid)
    _log.debug(f"Route rule: {route_rule}")

    filtered_rules = list(filter(lambda rule: rule.destination == cidr and rule.network_entity_id == ne_ocid,
                                 route_table.route_rules))

    if filtered_rules:
        return True
    else:
        return False


def construct_update_details(route_table, cidr, ne_ocid, action):
    route_rules = route_table.route_rules.copy()

    if action == 'create':
        route_rules.append(_construct_route_rule(cidr, ne_ocid))
    elif action == 'delete':
        filtered_rules = list(filter(lambda rule: rule.destination != cidr and rule.network_entity_id != ne_ocid,
                                     route_rules))

        route_rules = filtered_rules
    else:
        raise ValueError(f"Action {action} is not implemented yet")

    update_route_table_details = UpdateRouteTableDetails(
        route_rules=route_rules
    )
    _log.debug(f"Route table update details: {update_route_table_details}")

    return update_route_table_details


def _do_update_route_table(route_table_id, update_route_table_details):
    network_client = get_network_client()
    response = network_client.update_route_table(route_table_id, update_route_table_details=update_route_table_details)

    return response.data


def update_route_table(route_table, cidr, ne_ocid, action, dry_run):
    update_route_table_details = construct_update_details(route_table, cidr, ne_ocid, action)
    _log.debug(f"Route table update details: {update_route_table_details}")

    if not dry_run:
        response = _do_update_route_table(route_table.id, update_route_table_details)
        _log.debug(f"{response}")

        return response
    else:
        _log.debug("Dry run, just sit around")

        return {}



from os import environ, path
from oci import config, core, auth


def _get_oci_config():
    """
    To unify different tools (oci-cli, ortu, ansible, terraform), I'm using the following doc
    https://docs.oracle.com/en-us/iaas/Content/API/SDKDocs/clienvironmentvariables.htm#CLI_Environment_Variables
    :return: oci_config
    """
    oci_user = environ.get("OCI_CLI_USER")
    oci_fingerprint = environ.get("OCI_CLI_FINGERPRINT")
    oci_tenancy = environ.get("OCI_CLI_TENANCY")
    oci_region = environ.get("OCI_CLI_REGION")
    oci_key_file = environ.get("OCI_CLI_KEY_FILE")
    oci_key_content = environ.get("OCI_CLI_KEY_CONTENT")

    if oci_user and oci_fingerprint and oci_tenancy and oci_region and (oci_key_file or oci_key_content):
        oci_config = {
            "user": oci_user,
            "fingerprint": oci_fingerprint,
            "tenancy": oci_tenancy,
            "region": oci_region
        }

        if oci_key_file:
            oci_config["key_file"] = oci_key_file
        else:
            oci_config["key_content"] = oci_key_content

    else:
        oci_config_file = environ.get("OCI_CLI_CONFIG_FILE",
                                      path.join(path.expanduser("~"), ".oci", "config"))
        oci_config_profile = environ.get("OCI_CONFIG_PROFILE",
                                         environ.get("OCI_CLI_PROFILE", "DEFAULT"))

        oci_config = config.from_file(file_location=oci_config_file, profile_name=oci_config_profile)

    config.validate_config(oci_config)

    return oci_config


def get_network_client():
    oci_config = {}
    signer = None

    # https://docs.oracle.com/en-us/iaas/Content/API/SDKDocs/clienvironmentvariables.htm#environmentvariabletable
    auth_type = environ.get("OCI_CLI_AUTH", "api_key")
    if auth_type == "api_key":
        oci_config = _get_oci_config()
    elif auth_type == "instance_principal":
        signer = auth.signers.InstancePrincipalsSecurityTokenSigner()

    return core.VirtualNetworkClient(config=oci_config, signer=signer)



import argparse


def args_parser():
    parser = argparse.ArgumentParser(description='Updates OCI route table')
    parser.add_argument('action', type=str, default='create', nargs='?', const='create', choices=['create', 'delete'],
                        help="Action to do with route table. The default is to create rule based on specified params")

    parser.add_argument('--rt-ocid', type=str, action='store',
                        help='Route table OCI ID')

    parser.add_argument('--cidr', type=str, action='store',
                        help='Destination network in CIDR notation - xxx.xxx.xxx.xxx/xx')

    parser.add_argument('--ne-ocid', type=str, action='store',
                        help='Network entity OCI ID')

    parser.add_argument('--debug', default=False, action='store_true',
                        help='Shows detailed update process')

    parser.add_argument('--dry-run', default=False, action='store_true',
                        help='Dry run update. Useful with --debug')

    return parser.parse_args()



import logging
from sys import exit
from ortu.arguments import args_parser
from ortu.oci_route_table import get_route_table, check_route, update_route_table

_log = logging.getLogger(name=__name__)


def main():
    args = args_parser()

    if args.debug:
        level = getattr(logging, 'DEBUG', None)
    else:
        level = getattr(logging, 'INFO', None)

    logging.basicConfig(level=level)

    exit_code = 1
    try:
        if args.rt_ocid:
            route_table = get_route_table(args.rt_ocid)
            _log.debug(f"Route table: {route_table}")
        else:
            exit_code = 2
            raise ValueError("Route table OCID is required")

        if args.cidr and args.ne_ocid:
            is_route_present = check_route(route_table, args.cidr, args.ne_ocid)
            _log.debug(f"Is route present in route table?: {is_route_present}")
        else:
            exit_code = 2
            raise ValueError("Both cidr and network entity id are required")

        if (is_route_present and args.action == 'create') or (not is_route_present and args.action == 'delete'):
            _log.debug("Just sit around")
        else:
            update_route_table(route_table, args.cidr, args.ne_ocid, args.action, args.dry_run)

        exit_code = 0
    except Exception as e:
        print(f"Something went wrong: {e}. Try to --debug it")

    return exit_code


if __name__ == '__main__':
    exit(main())



from ortu.oci_route_table import construct_update_details
from oci.core.models import RouteTable, RouteRule


WO_CIDR_ROUTE_RULES = [
    RouteRule(
        destination='192.168.1.0/24',
        destination_type='CIDR_BLOCK',
        network_entity_id='ocid1.localpeeringgateway.oc1.region.lpgid'
    ),
    RouteRule(
        destination='all-fra-services-in-oracle-services-network',
        destination_type='SERVICE_CIDR_BLOCK',
        network_entity_id='ocid1.servicegateway.oc1.region.sgwid'
    )
]

WO_SERVICE_ROUTE_RULES = [
    RouteRule(
        destination='0.0.0.0/0',
        destination_type='CIDR_BLOCK',
        network_entity_id='ocid1.internetgateway.oc1.region.igid'
    ),
    RouteRule(
        destination='192.168.1.0/24',
        destination_type='CIDR_BLOCK',
        network_entity_id='ocid1.localpeeringgateway.oc1.region.lpgid'
    )
]

CIDR_ROUTE_RULES = [
    RouteRule(
        destination='192.168.1.0/24',
        destination_type='CIDR_BLOCK',
        network_entity_id='ocid1.localpeeringgateway.oc1.region.lpgid'
    ),
    RouteRule(
        destination='all-fra-services-in-oracle-services-network',
        destination_type='SERVICE_CIDR_BLOCK',
        network_entity_id='ocid1.servicegateway.oc1.region.sgwid'
    ),
    RouteRule(
        destination='0.0.0.0/0',
        destination_type='CIDR_BLOCK',
        network_entity_id='ocid1.internetgateway.oc1.region.igid'
    )
]

SERVICE_ROUTE_RULES = [
    RouteRule(
        destination='0.0.0.0/0',
        destination_type='CIDR_BLOCK',
        network_entity_id='ocid1.internetgateway.oc1.region.igid'
    ),
    RouteRule(
        destination='192.168.1.0/24',
        destination_type='CIDR_BLOCK',
        network_entity_id='ocid1.localpeeringgateway.oc1.region.lpgid'
    ),
    RouteRule(
        destination='all-fra-services-in-oracle-services-network',
        destination_type='SERVICE_CIDR_BLOCK',
        network_entity_id='ocid1.servicegateway.oc1.region.sgwid'
    )
]

WO_CIDR_ROUTE_TABLE = RouteTable(
    route_rules=WO_CIDR_ROUTE_RULES
)

WO_SERVICE_ROUTE_TABLE = RouteTable(
    route_rules=WO_SERVICE_ROUTE_RULES
)

ROUTE_TABLE = RouteTable(
    route_rules=SERVICE_ROUTE_RULES
)

CIDR = '0.0.0.0/0'
IG_ID = 'ocid1.internetgateway.oc1.region.igid'
SERVICE_CIDR = 'all-fra-services-in-oracle-services-network'
SG_ID = 'ocid1.servicegateway.oc1.region.sgwid'


def test_create_cidr():
    assert construct_update_details(WO_CIDR_ROUTE_TABLE, CIDR, IG_ID, 'create').route_rules == \
           CIDR_ROUTE_RULES


def test_create_service():
    assert construct_update_details(WO_SERVICE_ROUTE_TABLE, SERVICE_CIDR, SG_ID, 'create').route_rules == \
           SERVICE_ROUTE_RULES


def test_delete_cidr():
    assert construct_update_details(ROUTE_TABLE, CIDR, IG_ID, 'delete').route_rules == \
           WO_CIDR_ROUTE_RULES


def test_delete_service():
    assert construct_update_details(ROUTE_TABLE, SERVICE_CIDR, SG_ID, 'delete').route_rules == \
           WO_SERVICE_ROUTE_RULES
